from time import sleep

from scapy.all import *  # Scapy
from scapy.layers.dns import DNS, DNSRR
from scapy.layers.inet import IP, UDP
from scapy.layers.l2 import Ether

from dash import html, callback, dcc
import dash_bootstrap_components as dbc
from dash.dependencies import Input, Output, State
from dash import dash_table, no_update

import pandas as pd  # Pandas

from libratom.lib.pff import PffArchive

myMAC = get_if_hwaddr(conf.iface)  # moja MAC adresa
myIP = get_if_addr(conf.iface)  # moja IP adresa

PSTarchive = PffArchive('data/outlook.pst')  # objekt triedy PffArchive (libratom/lib/pff.py)

# ulozenie obsahu pst suboru do DataFrame
emails_data = pd.DataFrame(columns=['id', 'From', 'Subject', 'SubmitTime', 'Attachments'])
emails_data.set_index('id', inplace=True)
emails_data_id = 0  # id zaznamu

close_clicks = 0  # pomocna premenna


def extract_msg_content(msg):
    msg_content = ''
    
    if msg.plain_text_body:
        msg_content = msg.plain_text_body
    elif msg.html_body:
        msg_content = msg.html_body
    elif msg.rtf_body:
        try:
            msg_content = msg.rtf_body
        except UnicodeEncodeError:
            msg_content = 'Chyba kódovania!'
    else:
        msg_content = 'Nedostupný obsah správy!'

    return msg_content.decode('utf-8')


# vzhlad stranky
layout = dbc.Container([
    dbc.Row([
        html.H1('Sken 7 - Načítanie obsahu MS Otulook PST súboru')
    ]),
    dbc.Row([
        html.Table(
            style={'marginBottom': '20px'},
            children=[
                html.Tr(
                    children=[
                        html.Td(
                            style={'text-align': 'center', 'border-bottom': 'none',
                                   'border-right': '1px solid', 'width': '33%'},
                            children=[html.Div('Moja MAC adresa je:'),
                                      html.Div(className='ADResa', children=[myMAC])]),
                        html.Td(
                            style={'text-align': 'center', 'border-bottom': 'none',
                                   'border-right': '1px solid', 'width': '33%'},
                            children=[html.Div('Moja IP adresa je:'),
                                      html.Div(className='ADResa', children=[myIP])]),
                        html.Td(
                            style={'text-align': 'center', 'border-bottom': 'none', 'width': '33%'},
                            children=[html.Div('Testuje sa sieť:'),
                                      html.Div(className='ADResa', children=[myIP + '/24'])])
                    ]
                )
            ]
        )
    ]),
    dbc.Row([
		html.Div([  # dialog pre zobrazenie obsahu sprav a ich hlaviciek
			dbc.Modal([dbc.ModalHeader(html.Div(id='modal_header', children=[''])),
					   dbc.ModalBody(html.Div(id='modal_text', children=[''])),
                       dbc.ModalFooter(dbc.Button(
										'ZATVOR', id='close', n_clicks=0,
										style={'background-color': '#333333',
											   'color': '#ffffff',
                                               'border': '1px solid #888',
                                               'font-size': '11px',
                                               'font-weight': '600'})
                       )
                      ],
                      id='modal',
                      size='lg',
                      scrollable=True,
                      is_open=False
            )
        ]),
        html.Div(children=[html.Table(style={'marginBottom': '0px'},
                                      children=[
                                          html.Tr(children=[
                                              html.Td(style={'border-bottom': 'none'},
                                                      children=[html.Button(id='page7read',
                                                                            n_clicks=0,
                                                                            children=['PREČÍTAJ SÚBOR'],
                                                                            style={'margin-bottom': '10px',
                                                                                   'background-color': '#333333',
                                                                                   'color': '#ffffff'}
                                                                            )
                                                               ])
                                          ]
                                          )
                                     ])
                           ]),
        dcc.Input(id='page7pomocka', value='0', style={'display': 'none'}),
        html.Div(children=[html.H2('Obsah MS Outlook PST súboru')]),
        html.Div(children=[dash_table.DataTable(
            id='page7table1',
            fixed_rows={'headers': True, 'data': 0},
            columns=[{'id': 'From', 'name': 'Odosielateľ'},
                     {'id': 'Subject', 'name': 'Subjekt'},
                     {'id': 'SubmitTime', 'name': 'Dátum a čas odoslania'},
                     {'id': 'Attachments', 'name': 'Počet príloh'}],
            style_table={'border': 'grey solid',
                         'height': 600,
                         'margin-bottom': '10px'},
            style_header={'color': 'lightgrey',
                          'backgroundColor': 'rgb(30,30,30)',
                          'fontWeight': 'bold'},
            style_cell={'color': 'white',
                        'backgroundColor': 'rgb(50,50,50)',
                        'border': '0px solid',
                        'font-size': '1.2em',
                        'textAlign': 'center',
                        'min-width': '50px',
                        'maxWidth': '150px',
                        'whiteSpace' : 'normal',
                        'height' : 'auto'},
            style_as_list_view=True,
            style_data_conditional=[{'if': {'row_index': 'odd'},
                                     'backgroundColor': 'rgb(70,70,70)'},
                                    {'if': {'state': 'active'},
                                     'backgroundColor': 'rgba(0,116,217,0.3)',
                                     'border': '0px solid'}],
            page_action='none'),
            dcc.Interval(
                id='page7table1_interval',
                interval=5000,  # aktualizuje n_intervals kazdych 5s
                n_intervals=0
            )
        ])
    ])
])


# Callback funkcia pre tlacidlo - potrebna pre spravne zobrazovanie textu tlacidla pri vybere stranok
@callback(Input('page7read', 'n_clicks'),
          prevent_initial_call=True)
def update_page7read(v):
    global PSTarchive
    global emails_data_id

    for f in PSTarchive.folders():  # nacitanie udajov patriacich jednotlivym spravam
        if f.get_number_of_sub_messages()!=0:
            for msg in f.sub_messages:
				# https://github.com/libyal/libpff/blob/main/pypff/pypff_message.c
				# .local/lib/python3.10/site-packages/pypff.cpython-310-x86_64-linux-gnu.so
                submittime=msg.get_client_submit_time()
                submittime=submittime.strftime('%d.%m.%Y %H:%M:%S')
                
                emails_data.loc[emails_data_id]=(msg.get_sender_name(), 
												 msg.get_subject(),
												 submittime,
												 msg.get_number_of_attachments())
                emails_data_id+=1  # bol pridany novy zaznam


# Callback funkcia pre tabulku 1 - aktualizacia kazdych 5s
@callback(Output('page7table1', 'data'),
          [Input('page7table1_interval', 'n_intervals')])
def update_page7table1(n):
    return emails_data.to_dict('records')


# Callback funkcia pre tabulku 1 - otvorenie dialogu s obsahom spravy a headers
@callback([Output('modal', 'is_open'),
		   Output('modal_text', 'children'),
		   Output('modal_header', 'children')],
          [Input('page7table1', 'active_cell'), Input('close', 'n_clicks')],
          [State('modal', 'is_open')],
          prevent_initial_call=True)
def show_msg_content(active_cell, c, is_open):
    global PSTarchive
    global close_clicks

    if c > close_clicks:  # zatvorenie dialogu po stlaceni tlacidla ZATVOR
        close_clicks = c
        return [False, '', '']

    active_cell_row_index=0
    
    if active_cell:
        active_cell_row_index = active_cell['row']  # id aktivneho riadku
        #print(active_cell_row_index)

        id=0
        for f in PSTarchive.folders():  # vyber konkretnej spravy podla id aktivneho riadku
            if f.get_number_of_sub_messages() != 0:
                for msg in f.sub_messages:
                    if id==active_cell_row_index:
                        return [True,
								extract_msg_content(msg),
								msg.get_transport_headers()]  # obsah spravy a headers
                    id+=1

    return [False, '', '']
